/**
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */
package abstrata;

import java.awt.event.KeyEvent;
import java.util.ArrayList;
import javax.swing.JPanel;
import objetos.CanhaoDePlasma;
import objetos.Duke;
import objetos.Puffy;
import objetos.Tux;

/**
 * Classe Jogador contém o mínimo necessário para adicionar
 * um objeto da Tela e fazer com que este mova por toda sua
 * extensão.
 */
public abstract class Jogador extends Personagem {

    public ArrayList<CanhaoDePlasma> vetorCanhaoDePlasmaHorizontal; // vetor de disparos (horizontal) do personagem principal
    public ArrayList<CanhaoDePlasma> vetorCanhaoDePlasmaDiagonalSuperior; // vetor de disparos (diagonal superir) do personagem principal
    public ArrayList<CanhaoDePlasma> vetorCanhaoDePlasmaDiagonalInferior; // vetor de disparos (diagonal inferior) do personagem principal
    protected boolean invencibilidade; // para verificar se sofrerá colisões
    protected boolean ativarBombaAniquiladora; // para verificar se foi ativada
    protected boolean ativarRevitalizador; // para verificar se foi ativado
    private int velocidadeTiro = 2; // velocidade inicial do disparo
    public int velocidadePersonagem = 1; // velocidade inicial de movimentação
    public int numeroDeTiros; // limite de disparos horizontais
    public int numeroDeTiros1; // limite de disparos diagonal superior
    public int numeroDeTiros2; // limite de disparos diagonal inferior
    public int potenciaCanhao; // potência do canhão
    public boolean jogoAtivo; // correção de bug

    /**
     * É necessário informar o nome da imagem que será carregada no jogador
     * @param nomeImagem caminho onde está localizada a imagem
     * @param jp referencia ao JPanel pai onde este objeto jogador está inserido
     * @param px posicao X inicial onde o jogador irá aparecer
     * @param py posicao Y inicial onde o jogador irá aparecer
     */
    public Jogador(String nomeImagem, JPanel jp, int px, int py) {
        super(nomeImagem, jp, px, py);
        vetorCanhaoDePlasmaHorizontal = new ArrayList<CanhaoDePlasma>();
        vetorCanhaoDePlasmaDiagonalSuperior = new ArrayList<CanhaoDePlasma>();
        vetorCanhaoDePlasmaDiagonalInferior = new ArrayList<CanhaoDePlasma>();
        invencibilidade = false;
        ativarBombaAniquiladora = false;
        this.numeroDeTiros = 5;
        this.numeroDeTiros1 = 5;
        this.numeroDeTiros2 = 5;
        ativarRevitalizador = false;
        potenciaCanhao = 0;
        jogoAtivo = true;
    }

    /**
     * Para tratar a tecla que está pressionada. Enquanto a tecla de direção
     * estiver pressionada, o objeto será movido para a respectiva direção.
     * Neste método é alterado os atributos dirX e dirY que são usados pelo
     * método mover
     * @param evento tecla que foi pressionada
     */
    public void teclaPressionada(KeyEvent evento) {
        if (jogoAtivo == true) {
            // obtendo o código da tecla que foi pressionado
            int tecla = evento.getKeyCode();
            switch (tecla) {
                case KeyEvent.VK_LEFT:
                    dirX = -velocidadePersonagem; // movendo para a esquerda
                    break;
                case KeyEvent.VK_RIGHT:
                    dirX = velocidadePersonagem; // movendo para a direita
                    break;
                case KeyEvent.VK_UP:
                    dirY = -velocidadePersonagem; // movendo para cima
                    break;
                case KeyEvent.VK_DOWN:
                    dirY = velocidadePersonagem; // movendo para baixo
                    break;
                case KeyEvent.VK_C:
                    // dispara canhão de plasma do personagem principal
                    dispararCanhao();
                    break;
                case KeyEvent.VK_R: // ativando revitalizador
                    // uso do polimorfismo
                    if (this instanceof Duke) {
                        // olhar método
                        ((Duke) this).ativarRevitalizador(true);
                        // uso do polimorfismo
                    } else if (this instanceof Tux) {
                        // olhar método
                        ((Tux) this).ativarRevitalizador(true);
                    }
                    break;
                case KeyEvent.VK_E: // ativando escudo de força
                    // uso do polimorfismo
                    if (this instanceof Duke) {
                        // olhar método
                        ((Duke) this).ativarEscudoDeForca(true);
                        // uso do polimorfismo
                    } else if (this instanceof Puffy) {
                        // olhar método
                        ((Puffy) this).ativarEscudoDeForca(true);
                    }
                    break;
                case KeyEvent.VK_B: // ativando bomba aniquiladora
                    // uso do polimorfismo
                    if (this instanceof Puffy) {
                        // olhar método
                        ((Puffy) this).ativarBombaAniquiladora();
                        // uso do polimorfismo
                    } else if (this instanceof Tux) {
                        // olhar método
                        ((Tux) this).ativarBombaAniquiladora();
                    }
                    break;
            }
        }
    }

    /**
     * Para de mover o objeto na direção da tecla que fora solta.
     * @param evento tecla que foi solta
     */
    public void teclaSolta(KeyEvent evento) {
        if (jogoAtivo == true) {
            // obtendo o código da tecla que foi solta
            int tecla = evento.getKeyCode();
            switch (tecla) {
                case KeyEvent.VK_LEFT:
                    dirX = 0; // para de mover para  esquerda
                    break;
                case KeyEvent.VK_RIGHT:
                    dirX = 0; // para de mover para  a direita
                    break;
                case KeyEvent.VK_UP:
                    dirY = 0; // para de mover para  cima
                    break;
                case KeyEvent.VK_DOWN:
                    dirY = 0; // para de mover para baixo
                    break;
            }
        }
    }

    /**
     * Aumenta a velocidade do tiro
     * @param v incrementa a velocidade do tiro do personagem principal
     */
    public void incrementarVelocidadeTiro(int v) {
        this.velocidadeTiro += v;
    }

    /**
     * Aumenta a velocidade de movimentação
     * @param vp incrementa velocidade de movimentação do personagem principal
     */
    public void incrementarVelocidadePersonagem(int vp) {
        this.velocidadePersonagem += vp;
    }

    /**
     * Aumenta a quantidade de disparos possíveis horizontais
     * @param d incrementa o número de disparos horizontais
     */
    public void incrementarNumeroDeDisparosHorizontal(int d) {
        this.numeroDeTiros += d;
    }

    /**
     * Aumenta a quantidade de disparos possíveis diagonal superior
     * @param ds incrementa o número de disparos da diagonal superior
     */
    public void incrementarNumeroDeDisparosDiagonalSuperior(int ds) {
        this.numeroDeTiros1 += ds;
    }

    /**
     * Aumenta a quantidade de disparos possíveis diagonal inferior
     * @param di incrementa o número de disparos da daigonal inferior
     */
    public void incrementarNumeroDeDisparosDiagonalInferior(int di) {
        this.numeroDeTiros2 += di;
    }

    /**
     * Aumenta a potência do canhão de plasma
     * @param p aumenta a pontência do canhao de plasma
     */
    public void incrementarPotenciaCanhao(int p) {
        this.potenciaCanhao += p;
    }

    /**
     * Determina a velocidade do personagem
     * @param v define o valor da velocidade do personagem
     */
    public void setVelocidade(int v) {
        this.velocidadePersonagem = v;
    }

    /**
     * Determina a potência do canhão de plasma
     * @param p define o valor da pontência do canhao de plasma
     */
    public void setPotenciaCanhao(int p) {
        this.potenciaCanhao = p;
    }

    /**
     * Adiciona um objeto CanhaoDePlasma no vetorCanhaoDePlasma
     */
    public void dispararCanhao() {

        if (vetorCanhaoDePlasmaHorizontal.size() < numeroDeTiros) {
            vetorCanhaoDePlasmaHorizontal.add(new CanhaoDePlasma("/imagens/laser.png", this.tela, x + this.getLargura(), y + this.getAltura() / 2 - 7, velocidadeTiro + 1));
        }
        // caso a potência do canhão seja menor ou igual a 3, começa a disparar na diagonal superior
        if (potenciaCanhao >= 3) {
            if (vetorCanhaoDePlasmaDiagonalSuperior.size() < numeroDeTiros1) {
                vetorCanhaoDePlasmaDiagonalSuperior.add(new CanhaoDePlasma("/imagens/laser.png", this.tela, x + this.getLargura(), y + this.getAltura() / 2 - 7, velocidadeTiro + 1));
            }
        }
        // caso a potência do canhão seja igual a 4, começa a disparar na diagonal inferior
        if (vetorCanhaoDePlasmaDiagonalInferior.size() < numeroDeTiros2) {
            if (potenciaCanhao >= 4) {
                vetorCanhaoDePlasmaDiagonalInferior.add(new CanhaoDePlasma("/imagens/laser.png", this.tela, x + this.getLargura(), y + this.getAltura() / 2 - 7, velocidadeTiro + 1));
            }
        }
    }

    /**
     * Retorna o conteúdo do vetor que armazena os disparos (horizontal)
     * @return vetorCanhaoDePlasma conteúdo do vetor que armazena os disparos
     */
    public ArrayList<CanhaoDePlasma> obterCanhaoHorizontal() {
        return vetorCanhaoDePlasmaHorizontal;
    }

    /**
     * Retorna o conteúdo do vetor que armazena os disparos (diagonal superior)
     * @return vetorCanhaoDePlasmaDiagonalSuperior conteúdo do vetor que armazena os disparos
     */
    public ArrayList<CanhaoDePlasma> obterCanhaoDiagonalSuperior() {
        return vetorCanhaoDePlasmaDiagonalSuperior;

    }

    /**
     * Retorna o conteúdo do vetor que armazena os disparos (diagonal inferior)
     * @return vetorCanhaoDePlasmaDiagonalInferior conteúdo do vetor que armazena os disparos
     */
    public ArrayList<CanhaoDePlasma> obterCanhaoDiagonalInferior() {
        return vetorCanhaoDePlasmaDiagonalInferior;
    }

    /**
     * Ativa ou desativa a invencibilidade
     * @param i se for true, ativa a invencibilidade, false, desativa
     */
    public void setInvencibilidade(boolean i) {
        this.invencibilidade = i;
    }

    /**
     * Retorna o valor atual da invencibilidade
     * Se for falso, personagem principal sofrerá danos
     * @return invencibilidade  se for falso, personagem principal sofrerá danos
     */
    public boolean obterInvencibilidade() {
        return invencibilidade;
    }

    /**
     * Ativa ou desativa a bomba aniquiladora
     * @param b se for true, ativa a bomba, false, desativa
     */
    public void ativarBombaAniquiladora(boolean b) {
        this.ativarBombaAniquiladora = b;
    }

    /**
     * Retorna o estado da bomba aniquiladora - ativado/desativo
     * @return ativarBombaAniquiladora
     */
    public boolean obterAtivarBomba() {
        return ativarBombaAniquiladora;
    }
}
