/**
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */
package objetos;

import abstrata.Jogador;
import javax.swing.JPanel;
import interfaces.BombaAniquiladora;
import interfaces.EscudoDeForça;
import jogo.Jogo;

public class Puffy extends Jogador implements BombaAniquiladora, EscudoDeForça {

    private Jogo jogo;
    public long tempoInicial;

    public Puffy(String nomeImagem, JPanel jp, int px, int py, Jogo jogo) {
        super("/imagens/puffy.png", jp, px, py);
        this.jogo = jogo;
    }

    /**
     * Ao ser ativada, destrói todos os inimigos e respectivos disparos que estão na tela
     * Não se aplica ao chefão.
     * @param ativarBomba destrói todos os inimigos e respectivos disparos que estão na tela
     * exceto chefão
     */
    @Override
    public void ativarBombaAniquiladora() {
        // para garantir que seja ativada somente uma vez
        if (this.ativarBombaAniquiladora == false) {
            // se o poder do personagem principal estiver no máximo
            // a bomba aniquiladora é ativada
            int poder = new Integer(jogo.jProgressBarPoder.getValue());
            if (poder == 100) {
                // ativandoa bomba aniquilidora
                this.ativarBombaAniquiladora = true;
                // atualizando a barra de poder do personagem principal
                jogo.jProgressBarPoder.setValue(0);
            } else {
                this.ativarBombaAniquiladora = false;
            }
        }
    }

    /**
     * Ao ser ativado, provê invencibilidade ao personagem principal por aproxidamente 5 segundos
     * @param ativarEscudo provê invencibilidade ao personagem principal por aproxidamente 5 segundos
     */
    @Override
    public void ativarEscudoDeForca(boolean ativarEscudo) {
        // para garantir que seja ativado somente uma vez
        if (this.invencibilidade == false) {
            // se o poder do personagem principal for no mínimo 50 pontos,
            // o escudo de força é ativado
            int poder = new Integer(jogo.jProgressBarPoder.getValue());
            if (poder >= 50) {
                // ativando o escudo de força
                this.invencibilidade = true;
                // alterando a imagem do personagem para identificar a
                // ativação do escudo de força
                this.alterarImagem("/imagens/puffyescudo.png");
                // atualizando barra de poder do personagem principal
                jogo.jProgressBarPoder.setValue(poder - 50);
            }
        }
    }
}
